# pylint: skip-file
"""
///
/// @file MD10State.py
///
/// @brief AUTOGENERATED FILE! ONLY EDIT IF YOU ARE A MACHINE!
///
/// @core 0bad34709094958919bfe30e6c0ca13c7682c49a
///
/// @build df4ac791e9dd25df235f8ed979101d1d2e75b19b
///
/// @date 2021-12-16 14:53:15
///
/// @author Dephy, Inc.
"""
from ctypes import Structure, c_int


class MD10State(Structure):
    _pack_ = 1
    _fields_ = [
        ("rigid", c_int),
        ("id", c_int),
        ("state_time", c_int),
        ("accelx", c_int),
        ("accely", c_int),
        ("accelz", c_int),
        ("gyrox", c_int),
        ("gyroy", c_int),
        ("gyroz", c_int),
        ("mot_ang", c_int),
        ("mot_vel", c_int),
        ("mot_acc", c_int),
        ("mot_cur", c_int),
        ("mot_volt", c_int),
        ("batt_volt", c_int),
        ("batt_curr", c_int),
        ("temperature", c_int),
        ("status_mn", c_int),
        ("status_ex", c_int),
        ("status_re", c_int),
        ("genvar_0", c_int),
        ("genvar_1", c_int),
        ("genvar_2", c_int),
        ("genvar_3", c_int),
        ("genvar_4", c_int),
        ("genvar_5", c_int),
        ("genvar_6", c_int),
        ("genvar_7", c_int),
        ("genvar_8", c_int),
        ("genvar_9", c_int),
        ("genvar_10", c_int),
        ("genvar_11", c_int),
        ("genvar_12", c_int),
        ("genvar_13", c_int),
        ("genvar_14", c_int),
        ("ank_ang", c_int),
        ("ank_vel", c_int),
        ("shank_ang", c_int),
        ("shank_vel", c_int),
        ("global_shank_ang", c_int),
        ("mot_from_ank", c_int),
        ("ank_from_mot", c_int),
        ("trans_ratio", c_int),
        ("ank_torque", c_int),
        ("step_count", c_int),
        ("step_time", c_int),
        ("gait_state", c_int),
        ("intermediate_state", c_int),
        ("movement", c_int),
        ("SystemTime", c_int),
    ]
